% Clear and calculate.
close all;
clear;
drawnow;

if isoctave()
    rand('state', 123);  %#ok<RAND>
else
    reset(RandStream.getGlobalStream, 123);
end
N = 1e4;

% Environments.
w3D = gen_waves('3D', N, 20, 'Ell');
w2D = gen_waves('2D', N, 20, 'Ell');
w3D.Eth = w3D.Eth / sqrt(2);
w3D.Eph = w3D.Eph / sqrt(2);
w2D.Eth = w2D.Eth / sqrt(2);
w2D.Eph = w2D.Eph / sqrt(2);

% Positions.
p = gen_positions([0 0 0 0]);

% Far-fields.
aDip1 = gen_farfield('inc. electric dipole', 20);
aDip2 = rotate_farfield(aDip1, 0, 90, 0);
aCosn1 = gen_farfield('cosn 45', 20);
aCosn2 = rotate_farfield(aCosn1, 90, 0, 0);

% Plot far-fields.
figure; plot_farfield(aDip1);
figure; plot_farfield(aDip2);
figure; plot_farfield(aCosn1);
figure; plot_farfield(aCosn2);

% Calculate channel matrices.
tic;
H_3D = calc_waves(w3D, [aDip1 aDip2 aCosn1 aCosn2], p);
H_2D = calc_waves(w2D, [aDip1 aDip2 aCosn1 aCosn2], p);
toc;

% Calculate capacity.
cap_3D = capacity(H_3D);
cap_2D = capacity(H_2D);

% Calculate parameters cumulatively.
n_samples_list = unique(round(logspace(log10(10), log10(N), 50)));
med_3D = zeros(size(H_3D,1), numel(n_samples_list));
med_2D = zeros(size(H_2D,1), numel(n_samples_list));
adg_3D_dip = zeros(1, numel(n_samples_list));
adg_3D_cosn = zeros(1, numel(n_samples_list));
adg_2D_dip = zeros(1, numel(n_samples_list));
adg_2D_cosn = zeros(1, numel(n_samples_list));
cap_3D_dip = zeros(1, numel(n_samples_list));
cap_3D_cosn = zeros(1, numel(n_samples_list));
cap_2D_dip = zeros(1, numel(n_samples_list));
cap_2D_cosn = zeros(1, numel(n_samples_list));
for ii = 1:numel(n_samples_list)
    ns = n_samples_list(ii);

    % Calculate parameters.
    med_3D(:,ii) = mean(abs(H_3D(:,1,1,1:ns)).^2, 4);
    med_2D(:,ii) = mean(abs(H_2D(:,1,1,1:ns)).^2, 4);
    [~,~,adg_3D_dip(ii)] = adg_algorithms(H_3D(1:2,1,1,1:ns));
    [~,~,adg_3D_cosn(ii)] = adg_algorithms(H_3D(3:4,1,1,1:ns));
    [~,~,adg_2D_dip(ii)] = adg_algorithms(H_2D(1:2,1,1,1:ns));
    [~,~,adg_2D_cosn(ii)] = adg_algorithms(H_2D(3:4,1,1,1:ns));
    cap_3D_dip(ii) = mean(capacity(H_3D(1:2,1,1,1:ns)));
    cap_3D_cosn(ii) = mean(capacity(H_3D(3:4,1,1,1:ns)));
    cap_2D_dip(ii) = mean(capacity(H_2D(1:2,1,1,1:ns)));
    cap_2D_cosn(ii) = mean(capacity(H_2D(3:4,1,1,1:ns)));
end

% MED
figure;
semilogx(n_samples_list, 10*log10(med_3D), '.-'); hold on;
semilogx(n_samples_list, 10*log10(med_2D), '-');
axis([0 max(n_samples_list) -12 0]);
grid on;
title('Estimated Mean Effective Directivity');
xlabel('Cumulative Number of Samples [-]');
ylabel('Estimated MED [dB]');
legend('3D Env., Vertical Dipole', ...
       '3D Env., Horizontal Dipole', ...
       '3D Env., Pencil Beam 1', ...
       '3D Env., Pencil Beam 2', ...
       '2D Env., Vertical Dipole', ...
       '2D Env., Horizontal Dipole', ...
       '2D Env., Pencil Beam 1', ...
       '2D Env., Pencil Beam 2', ...
       'Location', 'southwest');

% ADG
figure;
semilogx(n_samples_list, 10*log10(adg_3D_dip)); hold on;
semilogx(n_samples_list, 10*log10(adg_3D_cosn));
semilogx(n_samples_list, 10*log10(adg_2D_dip));
semilogx(n_samples_list, 10*log10(adg_2D_cosn));
axis([0 max(n_samples_list) 0 12]);
grid on;
title('Estimated Apparent Diversity Gain');
xlabel('Cumulative Number of Samples [-]');
ylabel('Estimated Apparent Diversity Gain [dB]');
legend('3D Env., Dipoles', ...
       '3D Env., Pencil Beams', ...
       '2D Env., Dipoles', ...
       '2D Env., Pencil Beams', ...
       'Location', 'southwest');

% Capacity
figure;
semilogx(n_samples_list, cap_3D_dip); hold on;
semilogx(n_samples_list, cap_3D_cosn);
semilogx(n_samples_list, cap_2D_dip);
semilogx(n_samples_list, cap_2D_cosn);
axis([0 max(n_samples_list) 2 6]);
grid on;
title('Estimated Capacity (for SNR = 15 dB)');
xlabel('Cumulative Number of Samples [-]');
ylabel('Estimated Capacity [bits/s/Hz]');
legend('3D Env., Dipoles', ...
       '3D Env., Pencil Beams', ...
       '2D Env., Dipoles', ...
       '2D Env., Pencil Beams', ...
       'Location', 'southwest');
