function animate_cases(s1, w1, a1, p1, s2, w2, a2, p2, n_samples_list)
%ANIMATE_CASES Figure animation of two virmlab cases.
%
%   function animate_cases(s1, w1, a1, p1, s2, w2, a2, p2, n_samples_list)
%
%   Animate cases.

% By Ulf Carlberg.

% Calculate all the channel matrices.
H1 = calc_waves(w1, a1, p1);
H2 = calc_waves(w2, a2, p2);
size(H1)

% Define figure and pause function.
hf = figure('OuterPosition', figpos('big'));
set(hf, 'Toolbar', 'figure');
pause_figure_animation(hf, 'on');

% Always animate irx=itx=ifreq=1.
v1 = shiftdim(H1(1,1,:,:,1),2).';
v2 = shiftdim(H2(1,1,:,:,1),2).';

% Animation loop.
ns_last = 1;
for ii = 1:numel(n_samples_list)
    ns = n_samples_list(ii);

    subplot(2, 2, 1);
    plot_cdf(v1(1:ns,:));
    hold off;
    title('CDF');
    xlabel('Power [dB]');
    ylabel('CDF [-]');
    grid on;
    axis([-40 10 1e-3 1e0]);
    big_text(0.01, 0.98, [s1 ', N = ' num2str(ns)]);

    subplot(2, 2, 2);
    plot_cdf(v2(1:ns,:));
    hold off;
    title('CDF');
    xlabel('Power [dB]');
    ylabel('CDF [-]');
    grid on;
    axis([-40 10 1e-3 1e0]);
    big_text(0.01, 0.98, [s2 ', N = ' num2str(ns)]);

    subplot(2, 2, 3);
    y1 = mean(abs(v1(1:ns_last,:)).^2, 1);
    y2 = mean(abs(v1(1:ns,:)).^2, 1);
    semilogx([ns_last ; ns], 10*log10([y1 ; y2]), 'linewidth', 1);
    if ii == 1
        hold on;
        title('Estimated Mean Effective Directivity');
        xlabel('Cumulative Number of Samples [-]');
        ylabel('Estimated MED [dB]');
        xlim([1e2 max(n_samples_list)]);
        ylim([-9 0]);
        grid on;
        big_text(0.01, 0.98, s1);
    end
    set(gca, 'ColorOrderIndex', 1);

    subplot(2, 2, 4);
    y1 = mean(abs(v2(1:ns_last,:)).^2, 1);
    y2 = mean(abs(v2(1:ns,:)).^2, 1);
    semilogx([ns_last ; ns], 10*log10([y1 ; y2]), 'linewidth', 1);
    if ii == 1
        hold on;
        title('Estimated Mean Effective Directivity');
        xlabel('Cumulative Number of Samples [-]');
        ylabel('Estimated MED [dB]');
        xlim([1e2 max(n_samples_list)]);
        ylim([-9 0]);
        grid on;
        big_text(0.01, 0.98, s2);
    end
    set(gca, 'ColorOrderIndex', 1);
    
    drawnow;
    ns_last = ns;
end
pause_figure_animation(hf, 'off');

% Define callback commands for the Control Buttons.
st = dbstack; % to get name of the script that called this function
if length(st)<2 % called from command line
    edit_str = 'commandwindow;';
else
    edit_str = ['edit(''' st(end).file ''');'];
end
info_str = ['fprintf(''' virmlab() '''); commandwindow;'];
fig_str = sprintf('figure(''OuterPosition'', [%d %d %d %d]); ', ...
    figpos('middle'));
plot_w_str = sprintf([fig_str ...
    'subplot(1, 2, 1); plot_waves(%s); ' ...
    'subplot(1, 2, 2); plot_waves(%s); '], inputname(2), inputname(6));
plot_a_str = sprintf([fig_str ...
    'subplot(1, 2, 1); plot_farfield_inpos(%s,%s); ' ...
    'subplot(1, 2, 2); plot_farfield_inpos(%s,%s); '], inputname(3), ...
    inputname(4), inputname(7), inputname(8));
plot_p_str = sprintf([fig_str ...
    'subplot(1, 2, 1); plot_positions(%s); ' ...
    'subplot(1, 2, 2); plot_positions(%s); '], inputname(4), inputname(8));

% Define the Control Buttons in the figure window.
bt([10 185], plot_w_str, 'Waves', 'Show Waves/Clusters in Environment');
bt([10 150], plot_a_str, 'Antennas', 'Show Antenna Radiation Patterns');
bt([10 115], plot_p_str, 'Positions', 'Show Positions and Orientations');
bt([10 80], edit_str, 'Edit Input', 'Edit Input Script');
bt([10 45], info_str, 'Info', 'Information (Matlab Command Window)');
bt([10 10], 'close all;', 'Close All', 'Close all Figures');

end

%-------------------------------------------------------------------------------

function big_text(x, y, s)
text(x, y, s, 'BackgroundColor', [.6 .6 1.], 'FontSize', 14, ...
    'VerticalAlignment', 'top', 'Units', 'normalized', 'interpreter', 'none');
end

%-------------------------------------------------------------------------------

function bt(pos, call, name, tip)
uicontrol('Style', 'pushbutton', 'String', name, 'TooltipString', tip, ...
    'Position', [pos(1) pos(2) 70 25], 'Callback', call);
end
