function H = calc_coupling(a_rx, p_rx, a_tx, p_tx, interp_method)
%CALC_COUPLING Calculate coupling between two cases.
%
%   function H = calc_coupling(a_rx, p_rx, a_tx, p_tx, interp_method)
%
%   Calculate coupling from antenna(s) "a_tx" in position(s) "p_tx" to
%   antenna(s) "a_rx" in position(s) "p_rx". The interpolation method can be
%   selected for the function "interp2", default is 'linear'. The function
%   returns the calculated channel matrix in "H".
%
%   The output matrix "H" have the following dimensions:
%       dim 1: RX Ports (dimension 2 of p_rx)
%       dim 2: TX Ports (dimension 2 of p_tx)
%       dim 3: RX Positions (dimension 1 of p_rx)
%       dim 4: TX Positions (dimension 1 of p_tx)
%       dim 5: Nfreq, number of frequencies (defined in antennas)

% By Ulf Carlberg.

% This function is optimized for speed assuming that:
%   nrx, ntx is small (1-8)
%   N1 is big
%   N2 is big
%   Nfreq is small

% Optional argument.
if nargin<5, interp_method = 'linear'; end

% Find dimensions and init H-matrix.
assert(ndims(a_rx)<=2 && ndims(p_rx)<=2 && ndims(a_tx)<=2 && ndims(p_tx)<=2, ...
    'The number of dimensions must be 2 for antennas and positions.');
Nrx = size(p_rx.x, 2);
Ntx = size(p_tx.x, 2);
N1 = size(p_rx.x, 1);
N2 = size(p_tx.x, 1);
Nfreq = a_rx(1).Nfreq;
H = complex(zeros(Nrx, Ntx, N1, N2, Nfreq));

% Check frequencies.
freq = a_rx(1).freq;
freq_tol = 0.1;
for irx = 1:Nrx
    assert(numel(a_rx(irx).freq) == Nfreq, ...
        'All frequencies must be the same in "a_rx".');
    assert(max(abs(a_rx(irx).freq-freq)) < freq_tol, ...
        'All frequencies must be the same in "a_rx".');
end
for itx = 1:Ntx
    assert(numel(a_tx(itx).freq) == Nfreq, ...
        'All frequencies in "a_tx" must the same as "a_rx".');
    assert(max(abs(a_tx(itx).freq-freq)) < freq_tol, ...
        'All frequencies in "a_tx" must the same as "a_rx".');
end

% Wavelength lambda.
lambda = 3e8 ./ freq;
lambda(freq<0) = 1;

% Loop through receive antennas.
for irx = 1:Nrx

    % Position of rx antenna #i1.
    x1 = p_rx.x(:,irx);
    y1 = p_rx.y(:,irx);
    z1 = p_rx.z(:,irx);
    alpha1 = p_rx.alpha(:,irx);
    theta1 = p_rx.theta(:,irx);
    phi1 = p_rx.phi(:,irx);

    % Loop through transmit antennas.
    for itx = 1:Ntx

        % Position of tx antenna #i2.
        x2 = p_tx.x(:,itx)';
        y2 = p_tx.y(:,itx)';
        z2 = p_tx.z(:,itx)';
        alpha2 = p_tx.alpha(:,itx)';
        theta2 = p_tx.theta(:,itx)';
        phi2 = p_tx.phi(:,itx)';

        % Distance and angles from antenna #i1 to antenna #i2.
        [r, theta, phi] = xyz2rthetaphi(x2-x1, y2-y1, z2-z1);

        % Calculate coupling. The expression "Eth1.*Eth2 - Eph1.*Eph2" have the
        % minus sign because the phi-vectors have opposite directions of the two
        % antennas. Conjugations cancel each other. This is verified for linear
        % and circular polarizations in "calc_coupling_test.m".
        for ifreq = 1:Nfreq
            [Eth1, Eph1] = get_farfield(a_rx(irx), alpha1, theta1, phi1, ...
                theta, phi, interp_method, ifreq);
            [Eth2, Eph2] = get_farfield(a_tx(itx), alpha2, theta2, phi2, ...
                180-theta, phi+180, interp_method, ifreq);
            H(irx,itx,:,:,ifreq) = (Eth1.*Eth2 - Eph1.*Eph2) .* ...
                lambda(ifreq) .* exp(-2j*pi*r/lambda(ifreq)) ./ (4*pi*r);
        end
    end
end

% Tweak to avoid -Inf dB-values. Makes nicer plots.
min_dB = -300;
min_a = 10^(min_dB/20);
indx = abs(H) < min_a;
H(indx) = min_a * exp(1j*angle(H(indx)));

end
