function H = calc_rlos(a, N)
%CALC_RLOS Calculate received voltage in random line-of-sight (LOS).
%
%   function H = calc_rlos(a, N)
%
%   Calculate received voltage for antenna "a" from random line-of-sight waves
%   by using regular sampling of the far-field sphere. "N" is the number of
%   samples of the sphere (default 1000). Output is the received voltages for
%   linear, right hand circular, and left hand circular, respectively.
%
%   The output matrix "H" have the following dimensions:
%       dim 1: RX ports (numel of "a")
%       dim 2: 3 (linear/RHC/LHC)
%       dim 3: N
%       dim 4: 1
%       dim 5: Nfreq, number of frequencies (defined in antennas)

% By Ulf Carlberg.

% Default arguments.
if nargin<2 || isempty(N), N = 1000; end

% Regular sampling of sphere.
[theta, phi] = sph_regsamp(N);

% Define psi angle (the polarization angle for linear polarization). We want
% different values for the different incoming directions, and we want it to vary
% "irregular". Therefore, the golden ratio is used here.
dpsi = 90*(sqrt(5)-1)/2;
psi = dpsi*(1:N)';

% Initialize output matrix.
Nrx = numel(a);
Nfreq = a(1).Nfreq;
H = complex(zeros(Nrx, 3, N, 1, Nfreq));

% Octave can't handle 'spline' interpolation in interp2 for non-uniform grid.
% Let Octave resort to 'linear'.
if isoctave()
    interp_method = 'linear';
else
    interp_method = 'spline';
end

% Loop through antennas and frequencies.
for irx = 1:Nrx
    for ifreq = 1:Nfreq

        % Evaluate far-field function with the regular sampling.
        Eth = interp2(a(irx).phi, a(irx).theta, a(irx).Eth(:,:,ifreq), ...
            phi, theta, interp_method, NaN);
        Eph = interp2(a(irx).phi, a(irx).theta, a(irx).Eph(:,:,ifreq), ...
            phi, theta, interp_method, NaN);

        % Calculate the random LOS from linear/RHC/LHC polarization.
        H(irx,1,:,1,ifreq) = Eth.*cosd(psi) + Eph.*sind(psi);
        H(irx,2,:,1,ifreq) = (Eth - 1j*Eph) / sqrt(2);
        H(irx,3,:,1,ifreq) = (Eth + 1j*Eph) / sqrt(2);
    end
end

end
