function H = calc_waves(w, a, p, interp_method)
%CALC_WAVES Calculate received voltages on antenna ports.
%
%   function H = calc_waves(w, a, p, interp_method)
%
%   Calculate the port voltages "H" for incoming waves "w" and antenna "a" in 
%   position "p".
%
%   The interpolation method may be chosen by defining "interp_method". Default
%   is 'linear'. See the help section of interp2 for more information.
%
%   The output matrix "H" have the following dimensions:
%       dim 1: Nrx, number of antennas (dim 2 of "a" and "p")
%       dim 2: Ntx, number of transmitters (dim 3 of "w")
%       dim 3: N1, number of positions (dim 1 of "p")
%       dim 4: N2, number of waves (dim 1 of "w")
%       dim 5: Nfreq, number of frequencies (defined in "a" struct)

% By Ulf Carlberg.

% This function is optimized for speed assuming that:
%   nrx, ntx is small (1-8)
%   N1 is big
%   N2 is big
%   Nfreq is small

if nargin<4 || isempty(interp_method), interp_method = 'linear'; end

% Tolerance for rotation angle [deg].
tol = 0.01;

% Initialize output variables.
Nrx = size(a,2);
assert(p.Na==Nrx, '"a" and "p" dimension mismatch');
Ntx = w.Ntx;
N1 = p.N;
N2 = w.N;
Nfreq = a(1).Nfreq;
H = complex(zeros([Nrx Ntx N1 N2 Nfreq]));

% Unit vectors for direction of incoming plane waves.
[urx, ury, urz] = unit_vector_r(w.theta, w.phi);  % size [N2,Nw,Ntx]
urx = reshape(urx, [1 size(urx)]);  % size [1,N2,Nw,Ntx]
ury = reshape(ury, [1 size(ury)]);  % size [1,N2,Nw,Ntx]
urz = reshape(urz, [1 size(urz)]);  % size [1,N2,Nw,Ntx]

% Incoming waves may be specified with negative phi angles, but antennas are
% always defined in [0, 360] interval.
w_phi = mod(w.phi, 360);

% Loop through receive antennas.
for irx = 1:Nrx
    
    % Account for phase-change of incoming plane waves later.
    lambda = 3e8 ./ a(irx).freq;
    lambda(a(irx).freq<0) = 1;
    dot_prod = urx .* p.x(:,irx) + ury .* p.y(:,irx) + ...
        urz .* p.z(:,irx);  % size [N1,N2,Nw,Ntx]

    % Rotation of the far-field is time-consuming, so we should try to avoid it.
    solved_positions = false(N1, 1);
    while ~all(solved_positions)
        i1 = find(~solved_positions, 1, 'first');
        indx = ~solved_positions & ...
            abs(p.alpha(:,irx)-p.alpha(i1,irx)) < tol & ...
            abs(p.theta(:,irx)-p.theta(i1,irx)) < tol & ...
            abs(p.phi(:,irx)-p.phi(i1,irx)) < tol;
        solved_positions(indx) = true;
        arot = rotate_farfield(a(irx), p.alpha(i1,irx), p.theta(i1,irx), ...
            p.phi(i1,irx));
        %fprintf('Solving %d positions at once...\n', sum(indx));  % debug

        % Loop through transmit set of waves.
        for itx = 1:Ntx

            % Loop through frequencies.
            for ifreq = 1:Nfreq

                % Evaluate far-field function for each plane wave and calculate
                % the scalar product.
                E = interp2(arot.phi, arot.theta, arot.Eth(:,:,ifreq), ...
                    w_phi(:,:,itx), w.theta(:,:,itx), interp_method, NaN) .* ...
                    w.Eth(:,:,itx) + ...
                    interp2(arot.phi, arot.theta, arot.Eph(:,:,ifreq), ...
                    w_phi(:,:,itx), w.theta(:,:,itx), interp_method, NaN) .* ...
                    w.Eph(:,:,itx);  % size [N2,Nw]
                E = reshape(E, [1 size(E)]);  % size [1,N2,Nw]

                % Get final voltages in ports.
                phase_change = exp(-2j*pi*dot_prod/lambda(ifreq));
                H(irx,itx,indx,:,ifreq) = sum(E.*phase_change(indx,:,:,itx), 3);
            end
        end
    end
end

end
