function check_farfield(a, ifreq_list)
%CHECK_FARFIELD Check far-field data structure.
%
%   function check_farfield(a, ifreq_list)
%
%   Check far-field data structure. Plot far-field at "ifreq".

% By Ulf Carlberg.

if nargin<2, ifreq_list = []; end
tol_deg = 0.001;
tol_field_dB = -60;

% Check struct variables.
assert(strcmp(a.type,'farfield'), 'Wrong field "type".');
assert(all(size(a.freq)==[a.Nfreq 1]), 'Wrong field "Nfreq" or "freq".');
assert(all(size(a.theta)==[a.Nth 1]), 'Wrong field "Nth" or "theta".');
assert(all(size(a.phi)==[a.Nph 1]), 'Wrong field "Nph" or "phi".');
if a.Nfreq==1
    tmp = [a.Nth a.Nph];
else
    tmp = [a.Nth a.Nph a.Nfreq];
end
assert(all(size(a.Eth)==tmp), 'Wrong field "Eth".');
assert(all(size(a.Eph)==tmp), 'Wrong field "Eph".');

% Angles must start at 0, end at 180/360 and be strictly monotonic.
dtheta = diff(a.theta);
dphi = diff(a.phi);
assert(a.theta(1)==0, 'Theta must start with 0.');
assert(a.theta(end)==180, 'Theta must end with 180.');
assert(a.phi(1)==0, 'Phi must start with 0.');
assert(a.phi(end)==360, 'Phi must end with 360.');
assert(all(dtheta>0), 'Theta must be strictly montonic.');
assert(all(dphi>0), 'Phi must be strictly montonic.');

% It is ok to have non-equidistant angles. But we check it.
if any(abs(dtheta-dtheta(1))>tol_deg)
    disp('Angles theta is non-equidistant. (That is allowed.)');
end
if any(abs(dphi-dphi(1))>tol_deg)
    disp('Angles phi is non-equidistant. (That is allowed.)');
end

A = hypot(a.Eth, a.Eph);
A_max = max(A(:));

% Phi angle 0 and 360 must have same fields values.
tmp1 = abs(a.Eth(:,1,:) - a.Eth(:,end,:));
tmp2 = abs(a.Eph(:,1,:) - a.Eph(:,end,:));
E_max = max([tmp1(:) ; tmp2(:)]);
assert(20*log10(E_max/A_max)<=tol_field_dB, ...
    'Field values at phi=0 and phi=360 does not match.');

for ii = ifreq_list(:)'
    figure;

    subplot(2,2,1);
    surf(a.phi, a.theta, abs(a.Eth(:,:,ii)));
    colormap(jet(256));
    caxis([0 A_max]);
    shading interp;
    set(gca, 'XTick', a.phi(1):90:a.phi(end));
    set(gca, 'YTick', a.theta(1):45:a.theta(end));
    title('Eth');
    xlabel('phi [deg]');
    ylabel('theta [deg]');

    subplot(2,2,2);
    surf(a.phi, a.theta, abs(a.Eph(:,:,ii)));
    colormap(jet(256));
    caxis([0 A_max]);
    shading interp;
    set(gca, 'XTick', a.phi(1):90:a.phi(end));
    set(gca, 'YTick', a.theta(1):45:a.theta(end));
    title('Eph');
    xlabel('phi [deg]');
    ylabel('theta [deg]');

    subplot(2,2,3);
    surf(a.phi, a.theta, hypot(a.Eth(:,:,ii),a.Eph(:,:,ii)));
    colormap(jet(256));
    caxis([0 A_max]);
    shading interp;
    set(gca, 'XTick', a.phi(1):90:a.phi(end));
    set(gca, 'YTick', a.theta(1):45:a.theta(end));
    title('W');
    xlabel('phi [deg]');
    ylabel('theta [deg]');

    subplot(2,2,4);
    plot_farfield(a, ii);
end

% Check north pole and south pole.
cos_phi = cosd(a.phi)';
sin_phi = sind(a.phi)';
np_max_error = 0;
sp_max_error = 0;
for ifreq = 1:a.Nfreq
    
    % North pole values should be same.
    ExPrim = a.Eth(1,:,ifreq).*cos_phi - a.Eph(1,:,ifreq).*sin_phi;
    EyPrim = a.Eth(1,:,ifreq).*sin_phi + a.Eph(1,:,ifreq).*cos_phi;
    tmp = max(hypot(ExPrim(1)-ExPrim, EyPrim(1)-EyPrim));
    np_max_error = max([np_max_error tmp]);
    
    % South pole values should be same.
    ExPrim = a.Eth(end,:,ifreq).*cos_phi - a.Eph(end,:,ifreq).*sin_phi;
    EyPrim = a.Eth(end,:,ifreq).*sin_phi + a.Eph(end,:,ifreq).*cos_phi;
    tmp = max(hypot(ExPrim(1)-ExPrim, EyPrim(1)-EyPrim));
    sp_max_error = max([sp_max_error tmp]);
end
fprintf('Max north, south pole error: %.1f dB, %.1f dB\n', ...
    20*log10(np_max_error/A_max), 20*log10(sp_max_error/A_max));

end
