function a = gen_farfield(ant_type, Nth, Nph)
%GEN_FARFIELD Generate far-field of (incremental) dipoles, isotropic antennas, Huygens sources, cosn.
%
%   function a = gen_farfield(ant_type, ntheta, nphi)
%
%   Generate antenna far-field of antenna type "ant_type". Return a
%   structure. "ntheta" and "nphi" are optional arguments specifying the
%   number of samples for the theta- and phi-angles.
%
%   "ant_type"                     Description
%   ---------------------------------------------------------------------------
%   'isotropic theta'            - isotropic antenna, theta-polarized
%   'isotropic phi'              - isotropic antenna, phi-polarized
%   'inc. electric monopole'     - z-directed, radiating for z>=0
%   'inc. electric dipole'       - z-directed
%   'inc. magnetic dipole'       - z-directed
%   'inc. magnetic dipole on GP' - z-directed
%   'inc. Huygens source'        - radiating in z-direction, x-polarized
%   'inc. Huygens source RHC'    - radiating in z-direction, RHC-polarized
%   'inc. Huygens source LHC'    - radiating in z-direction, LHC-polarized
%   'electric dipole'            - z-directed
%   'magnetic dipole'            - z-directed
%   'cosn X'                     - Pencil beam of "cos(theta/2)^n"-shape,
%                                  x-polarized, with "X" half beam width
%                                  in degrees (at -3 dB), e.g., 'cosn 40'
%                                  gives a pencil beam with 40 degrees
%                                  half beam width at the -3 dB level.
%   'cosn X Y'                   - X = half beam width E-plane
%                                  Y = half beam width H-plane
%
% References:
% [1] Foundations of Antennas - A Unified Approach, Per-Simon Kildal,
%     Studentlitteratur 2000.

% By Ulf Carlberg.

% Define optional arguments.
if nargin<2 || isempty(Nth), Nth = 19; end
if nargin<3 || isempty(Nph), Nph = 2*(Nth-1)+1; end

% Allow "incremental" instead of "inc.".
ant_type = strrep(ant_type, 'incremental', 'inc.');

% Save antenna type in structure.
a.type = 'farfield';
a.name = ant_type;
a.actions = {['gen_farfield(' function_call_str({ant_type, Nth, Nph}) ')']};
a.freq = -1;  % -1 indicates any frequency
a.Nth = Nth;
a.Nph = Nph;
a.Nfreq = numel(a.freq);

% Make a grid of theta and phi angles.
if strcmpi(ant_type, 'inc. electric monopole')
    a.theta = [linspace(0, 90, ceil(Nth/2)) ...
        linspace(90.0001, 180, Nth-ceil(Nth/2))]';
else
    a.theta = linspace(0, 180, Nth)';
end
a.phi = linspace(0, 360, Nph)';
[theta2D, phi2D] = ndgrid(a.theta, a.phi);

% Define radiation pattern.
Eth = zeros(Nth, Nph);
Eph = zeros(Nth, Nph);
if strcmpi(ant_type, 'isotropic theta')
    Eth = ones(Nth, Nph);
elseif strcmpi(ant_type, 'isotropic phi')
    Eph = ones(Nth, Nph);
elseif strcmpi(ant_type, 'inc. electric monopole')
    Eth = (theta2D<=90) .* sqrt(3) .* sind(theta2D);
elseif strcmpi(ant_type, 'inc. electric dipole')
    Eth = sqrt(1.5) * sind(theta2D);
elseif strcmpi(ant_type, 'inc. magnetic dipole')
    Eph = sqrt(1.5) * sind(theta2D);
elseif strcmpi(ant_type, 'inc. Huygens source')  % (='cosn' for n=2)
    ExPrim = sqrt(3) * cosd(theta2D/2).^2;
    Eth = cosd(phi2D) .* ExPrim;
    Eph = -sind(phi2D) .* ExPrim;
elseif strcmpi(ant_type, 'inc. Huygens source theta')
    Eth = sqrt(3) * cosd(theta2D/2).^2;
elseif strcmpi(ant_type, 'inc. Huygens source phi')
    Eph = sqrt(3) * cosd(theta2D/2).^2;
elseif strcmpi(ant_type, 'inc. Huygens source RHC')
    ExPrim = sqrt(3) * cosd(theta2D/2).^2 / sqrt(2);
    EyPrim = -1j * ExPrim;
    Eth = cosd(phi2D).*ExPrim + sind(phi2D).*EyPrim;
    Eph = -sind(phi2D).*ExPrim + cosd(phi2D).*EyPrim;
elseif strcmpi(ant_type, 'inc. Huygens source LHC')
    ExPrim = sqrt(3) * cosd(theta2D/2).^2 / sqrt(2);
    EyPrim = 1j * ExPrim;
    Eth = cosd(phi2D).*ExPrim + sind(phi2D).*EyPrim;
    Eph = -sind(phi2D).*ExPrim + cosd(phi2D).*EyPrim;
elseif strcmpi(ant_type, 'electric dipole')
    Eth = 1.28098527 * cosd(90*cosd(theta2D)) ./ sind(theta2D);
    Eth(1,:) = 0;
    Eth(end,:) = 0;
elseif strcmpi(ant_type, 'magnetic dipole')
    Eth = zeros(Nth, Nph);
    Eph = 1.28098527 * cosd(90*cosd(theta2D)) ./ sind(theta2D);
    Eph(1,:) = 0;
    Eph(end,:) = 0;
elseif strncmpi(ant_type, 'cosn ', 5) % cos(theta/2)^n
    half_beamwidth = sscanf(ant_type(6:end),'%f');
    assert(numel(half_beamwidth)>=1 && numel(half_beamwidth)<=2 && ...
        all(half_beamwidth>=5) && all(half_beamwidth<180), ...
        'The parameter(s) "half_beamwidth" is wrong.');
    ne = log(sqrt(0.5)) / log(cosd(half_beamwidth(1)/2));
    nh = log(sqrt(0.5)) / log(cosd(half_beamwidth(end)/2));
    D0 = 2*(ne+1)*(nh+1) / (ne+nh+2);
    Eth = sqrt(D0) * cosd(phi2D) .* cosd(theta2D/2).^ne;
    Eph = -sqrt(D0) * sind(phi2D) .* cosd(theta2D/2).^nh;
else
    error('Unknown antenna type.');
end

% Avoid exactly zero fields.
min_dB = -200;
min_a = 10^(min_dB/20);
indx = abs(Eth) < min_a;
Eth(indx) = min_a * exp(1j*angle(Eth(indx)));
indx = abs(Eph) < min_a;
Eph(indx) = min_a * exp(1j*angle(Eph(indx)));

% Save field in antenna structure.
a.Eth = single(Eth);  % single precision is more than enough and saves memory
a.Eph = single(Eph);

end
