function p = gen_positions(x, y, z, alpha, theta, phi)
%GEN_POSITIONS Generate positions (spatial translation and rotation) by expansion of input matrices.
%
%   function p = gen_positions(x, y, z, alpha, theta, phi)
%
%   Generates positions and direction angles in the structure "p". The input
%   matrices is automatically expanded so that all have the same size.
%   * Dimension 1 (rows) corresponds to when the DUT is moving around.
%   * Dimension 2 (columns) corresponds to the position of the different 
%     antennas for the DUT.
%
%   Examples:
%
%   * A single-port antenna in a fixed position (origin), no rotation:
%     "gen_positions(0, 0, 0, 0, 0, 0)"
%
%   * A single-port antenna moved along the x-axis and rotated 90 degrees by 
%     the theta-angle:
%     "gen_positions((1:100)', 0, 0, 0, 90, 0)"
%
%   * A four-port antenna, where each antenna is separated 0.2 length units 
%     (along the y-axis) and placed in a fixed position:
%     "gen_positions(0, [-0.3 -0.1 0.1 0.3], 0, 0, 0, 0)"

% By Ulf Carlberg.

% Optional arguments.
if nargin<2, y = 0; end
if nargin<3, z = 0; end
if nargin<4, alpha = 0; end
if nargin<5, theta = 0; end
if nargin<6, phi = 0; end

% All input variables should be (maximum) 2-dimensional.
max_ndim = max([ndims(x) ndims(y) ndims(z) ...
    ndims(alpha) ndims(theta) ndims(phi)]);
assert(max_ndim==2, 'All inputs must be maximum 2-dimensional.');

% The new size for all output matrices are the maximum size in each dimension of
% all the input matrices.
new_size = ones(1, max_ndim);
for dim = 1:max_ndim
    new_size(dim) = max([size(x,dim) size(y,dim) size(z,dim) ...
        size(alpha,dim) size(theta,dim) size(phi,dim)]);
end

% Save matrix size in output structure.
p.N = new_size(1);
p.Na = new_size(2);

% Expand coordinate matrices.
p.x = xmat(x, new_size);
p.y = xmat(y, new_size);
p.z = xmat(z, new_size);

% Expand direction angles matrices.
p.alpha = xmat(alpha, new_size);
p.theta = xmat(theta, new_size);
p.phi = xmat(phi, new_size);

end
