function w = gen_waves(env_type, N, Nw, pol_type, K_dB, Ntx)
%GEN_WAVES Generate multi-path environment by incident random plane waves.
%
%   function w = gen_waves(env_type, N, Nw, pol_type, K_dB, Ntx)
%
%   Generate N*Nw*Ntx plane waves.
%   env_type    - environment type:
%                '2D' means waves only from azimuth
%                '3D' means waves isotropically distributed
%                'Icosahedron' means 20 waves on each face of an icosahedron
%                [theta_min theta_max phi_min phi_max] means limited angles
%   N          - number of realizations
%   Nw         - number of waves per realization
%   pol_type   - polarization type 'LP' (default) / 'RHC' / 'LHC' / 'ELL' / 
%                '2LP' / '2CP' (2LP and 2CP means orthogonal and Ntx = 2.) /
%                'theta' / 'phi'.
%   K_dB       - Rician K-factor in [dB]. Optional, leave undefined or empty for
%                Rayleigh fading.
%   Ntx        - number of transmitters (dimension 3)

% By Ulf Carlberg.

% Define optional arguemnts.
if nargin<4 || isempty(pol_type), pol_type = 'LP'; end
if nargin<5, K_dB = []; end
if nargin<6 || isempty(Ntx)
    if pol_type(1) == '2'
        Ntx = 2;
    else
        Ntx = 1;
    end
end

% Check arguments.
theta_min = 0;
theta_max = 180;
phi_min = 0;
phi_max = 360;
if isa(env_type, 'double')
    theta_min = env_type(1);
    theta_max = env_type(2);
    phi_min = env_type(3);
    phi_max = env_type(4);
elseif strcmpi(env_type, '3D')
    % pass
elseif strcmpi(env_type, '2D')
    theta_min = 90;
    theta_max = 90;
elseif strcmpi(env_type, 'Icosahedron')
    assert(Nw==20, 'Nw out of range');
    assert(Ntx==1, 'Ntx out of range');
    assert(isempty(K_dB), 'K_dB cannot be defined for Icosahedron environment');
else
    error('Unknown environment type.');
end
if strcmpi(pol_type, '2LP') || strcmpi(pol_type, '2CP')
    assert(Ntx==2, 'Ntx out of range');
end
if ~isempty(K_dB)
    assert(Nw>=2, 'N must be at least 2 for Rician fading');
end

% Save input variables in output structure.
w.env_type = env_type;
w.N = N;  % number of realizations
w.Nw = Nw;  % number of waves per realization
w.Ntx = Ntx;  % number of transmitters
w.pol_type = pol_type;
w.K_dB = K_dB;
w.theta_min = theta_min;
w.theta_max = theta_max;
w.phi_min = phi_min;
w.phi_max = phi_max;

% Define wave directions.
if strcmpi(env_type, 'Icosahedron')
    gr = (1+sqrt(5))/2;  % golden ratio
    gi = 1/gr;  % golden ratio inverse
    coord = [ ...
         -1  -1  -1 ;  -1  -1  1 ; -1   1  -1 ; -1  1  1 ; ...
          1  -1  -1 ;   1  -1  1 ;  1   1  -1 ;  1  1  1 ; ...
          0 -gi -gr ;   0 -gi gr ;  0  gi -gr ;  0 gi gr ; ...
        -gi -gr   0 ; -gi  gr  0 ; gi -gr   0 ; gi gr  0 ; ...
        -gr   0 -gi ; -gr   0 gi ; gr   0 -gi ; gr  0 gi];  % from Wikipedia
    [~, theta, phi] = xyz2rthetaphi(coord(:,1), coord(:,2), coord(:,3));
    w.theta = repmat(theta', N, 1);
    w.phi = repmat(phi', N, 1);
else
    w.theta = acosd( cosd(theta_min) + (cosd(theta_max)-cosd(theta_min)) * ...
        rand(N,Nw,Ntx) );
    w.phi = phi_min + (phi_max-phi_min)*rand(N,Nw,Ntx);
end

% If we use Rician fading or only have one incoming wave, this first incoming 
% wave is a dominant LOS-wave. Then it must come from the same direction for all
% TX cases.
if ~isempty(K_dB) || Nw == 1
    for itx = 2:Ntx
        w.theta(:,1,itx) = w.theta(:,1,1);
        w.phi(:,1,itx) = w.phi(:,1,1);
    end
end

% Define amplitude density and phase of incoming waves.
amp = complex(randn(N,Nw,Ntx), randn(N,Nw,Ntx));
if strcmpi(env_type, 'Icosahedron')  % special normalization
    pow = sum(abs(amp).^2, 1);
    amp = amp .* repmat(sqrt(N./(Nw*pow)), N, 1);
elseif ~isempty(K_dB)  % Rician fading
    K = 10.^(K_dB/10);  % convert dB -> power ratio
    pow_los = abs(amp(:,1,:)).^2;
    pow_nonlos = sum(abs(amp(:,2:Nw,:)).^2, 2);
    amp(:,1,:) = amp(:,1,:) ./ sqrt((1+1/K)*pow_los);
    amp(:,2:Nw,:) = amp(:,2:Nw,:) ./ repmat(sqrt((1+K)*pow_nonlos), 1, Nw-1);
else  % normalize so that the sum of power densities for each realization is 1
    pow = sum(abs(amp).^2, 2);
    amp = amp ./ repmat(sqrt(pow), 1, Nw);
end

% If RIMP (many waves) normalize to 2. If RLOS (one wave) keep normalization.
% TODO: Is this really good?
if Nw > 1, amp = amp * sqrt(2); end

% Normalize with number of transmit antennas.
amp = amp / sqrt(Ntx);

% Define polarizations.
if strcmpi(pol_type, 'LP') || strcmpi(pol_type, 'Ell') || ...
        strcmpi(pol_type, '2LP')
    if strcmpi(env_type, 'Icosahedron')
        psi = repmat(360*rand(1,Nw,Ntx), N, 1);
    elseif strcmpi(pol_type, '2LP')
        psi = 360*rand(N,Nw,1);
        psi(:,:,2) = psi + 90;
    else
        psi = 360*rand(N,Nw,Ntx);
    end
    Eth = amp .* cosd(psi);
    Eph = amp .* sind(psi);
    if strcmpi(pol_type, 'Ell')
        Eth = Eth .* exp(2j*pi*rand(N,Nw,Ntx));
        Eph = Eph .* exp(2j*pi*rand(N,Nw,Ntx));
    end
elseif strcmpi(pol_type, 'theta')
    Eth = amp;
    Eph = zeros(size(amp));
elseif strcmpi(pol_type, 'phi')
    Eth = zeros(size(amp));
    Eph = amp;
elseif strcmpi(pol_type, 'RHC')
    Eth = amp/sqrt(2);
    Eph = -1j*amp/sqrt(2);
elseif strcmpi(pol_type, 'LHC')
    Eth = amp/sqrt(2);
    Eph = 1j*amp/sqrt(2);
elseif strcmpi(pol_type, '2CP')
    Eth = amp/sqrt(2);
    Eph = -1j*amp/sqrt(2);
    Eph(:,:,2) = -Eph(:,:,2);
else
    error('Unknown polarization type.');
end
w.Eth = complex(Eth);
w.Eph = complex(Eph);

end
