function AR_dB = get_ar(a, theta, phi, freq)
%GET_AR Get axial ratio (AR) from antenna structure far-field.
%
%   function AR_dB = get_ar(a, theta, phi, freq)
%
%   Get axial ratio (AR) in decibel [dB] from far-field "a" in
%   directions "theta" and "phi" for frequencies "freq". The axial ratio is
%   defined as the major axis divided by the minor axis of the polarization 
%   ellipse. An ideal linear polarization will have an axial ratio of 300 dB and
%   an ideal circular polarization will have an axial ratio of 0 dB.

% By Ulf Carlberg.

if nargin<2, theta = []; end
if nargin<3, phi = []; end
if nargin<4, freq = []; end

% Get components.
[Eth, Eph] = get_farfield_components(a, theta, phi, freq);

% Calculate axial ratio (major axis / minor axis of polarization ellipse).
% Formulas from: https://en.wikipedia.org/wiki/Elliptical_polarization
theta_rad = atan(abs(Eth./Eph));
beta_rad = angle(Eth) - angle(Eph);
tmp = sqrt( 1 - sin(2*theta_rad).^2 .* sin(beta_rad).^2 );
AR_dB = 10*log10( (1+tmp) ./ (1-tmp) );

% Limit values to 300 dB because "Inf" can mess up plots.
AR_dB = min(AR_dB, 300);

end
