function [Eth, Eph] = get_farfield(a, alpha, theta, phi, theta2D, phi2D, interp_method, ifreq)
%GET_FARFIELD Get far-field from antenna structure without explicit rotation.
%
%   function [Eth, Eph] = get_farfield(a, alpha, theta, phi, ...
%       theta2D, phi2D, interp_method, ifreq)
%
%   Calculate far-field in directions given by the angles "theta2D" and "phi2D"
%   from antenna structure "a", zyz-rotated with angles "alpha", "theta", and
%   "phi". The far-field is not rotated explicitly, for improved execution
%   speed.

% By Ulf Carlberg.

% Optional argument.
if nargin<7 || isempty(interp_method), interp_method = 'linear'; end
if nargin<8 || isempty(ifreq), ifreq = 1; end

% Calculate the spherical unit vectors in the directions given by "theta2D" and
% "phi2D".
[uRx, uRy, uRz] = unit_vector_r(theta2D, phi2D);
[uTx, uTy, uTz] = unit_vector_theta(theta2D, phi2D);
[uPx, uPy, ~] = unit_vector_phi(phi2D);

% Rotate the unit vectors R backwards to get the new "theta2Drot" and "phi2Drot"
% angles of output antenna far-field.
[uRx2, uRy2, uRz2] = zyzrot(uRx, uRy, uRz, -phi, -theta, -alpha);
[~, theta2Drot, phi2Drot] = xyz2rthetaphi(uRx2, uRy2, uRz2);

% Calculate unit vectors in new directions.
[uTx2, uTy2, uTz2] = unit_vector_theta(theta2Drot, phi2Drot);
[uPx2, uPy2, uPz2] = unit_vector_phi(phi2Drot);

% Rotate the new unit vectors in forward direction.
[uTx2, uTy2, uTz2] = zyzrot(uTx2, uTy2, uTz2, alpha, theta, phi);
[uPx2, uPy2, uPz2] = zyzrot(uPx2, uPy2, uPz2, alpha, theta, phi);

% Avoid Octave bug?
octave_bug_work_around = numel(phi2Drot)==1 && isoctave();
if octave_bug_work_around
    phi2Drot = repmat(phi2Drot, 1, 2);
    theta2Drot = repmat(theta2Drot, 1, 2);
end

% Calculate the far-fields.
Eth1 = interp2(a.phi, a.theta, a.Eth(:,:,ifreq), phi2Drot, theta2Drot, ...
    interp_method, NaN);
Eph1 = interp2(a.phi, a.theta, a.Eph(:,:,ifreq), phi2Drot, theta2Drot, ...
    interp_method, NaN);

% Avoid Octave bug?
if octave_bug_work_around
    Eth1 = Eth1(1);
    Eph1 = Eph1(1);
end

% Project the far-field defined with the rotated unit vectors onto the proper
% unit vectors.
Eth = Eth1 .* (uTx.*uTx2 + uTy.*uTy2 + uTz.*uTz2) + ...
      Eph1 .* (uTx.*uPx2 + uTy.*uPy2 + uTz.*uPz2);
Eph = Eth1 .* (uPx.*uTx2 + uPy.*uTy2) + ...
      Eph1 .* (uPx.*uPx2 + uPy.*uPy2);

end
