function [Eth, Eph] = get_farfield_components(a, theta, phi, freq)
%GET_FARFIELD_COMPONENTS Get far-field components from antenna structure far-field.
%
%   function [Eth, Eph] = get_farfield_components(a, theta, phi, freq)
%
%   Get components from far-field in antenna structure "a" in directions 
%   "theta" and "phi" for frequency "freq". All three last elements are
%   optional, and the default is all available data.

% By Ulf Carlberg.

% Define optional arguments.
if nargin<2 || isempty(theta), theta = a.theta; end
if nargin<3 || isempty(phi), phi = a.phi; end
if nargin<4 || isempty(freq), freq = a.freq; end
interp_method = 'linear';

[theta3D, phi3D, freq3D] = ndgrid(theta, phi, freq);

% Make theta in interval [0,180] and phi in interval [0,360].
theta3D = mod(theta3D, 360);
indx = theta3D > 180;
theta3D(indx) = 360 - theta3D(indx);
phi3D(indx) = phi3D(indx) + 180;
phi3D = mod(phi3D, 360);

% Calculate the field.
if numel(a.freq)==1
    Eth = interp2(a.phi, a.theta, a.Eth, phi3D, theta3D, ...
        interp_method, NaN);
    Eph = interp2(a.phi, a.theta, a.Eph, phi3D, theta3D, ...
        interp_method, NaN);
    Eth = repmat(Eth, [1 1 numel(freq)]);
    Eph = repmat(Eph, [1 1 numel(freq)]);
    Eth(:,:,freq~=a.freq) = NaN;
    Eph(:,:,freq~=a.freq) = NaN;
else
    Eth = interp3(a.phi, a.theta, a.freq, a.Eth, phi3D, theta3D, freq3D, ...
        interp_method, NaN);
    Eph = interp3(a.phi, a.theta, a.freq, a.Eph, phi3D, theta3D, freq3D, ...
        interp_method, NaN);
end
Eth(indx) = -Eth(indx);
Eph(indx) = -Eph(indx);

end
