function b = isodd(x)
%ISODD True for odd numbers (raise error if wrong input).
%
%   function b = isodd(x)
%
%   Return logical true for odd numbers. Raise error if:
%   * too large or small to determine if it is odd or even (this includes -Inf
%     and +Inf), or
%   * contains a fraction (e.g. isodd(1.5) raises an error), or
%   * NaN (not-a-number).
%
%   See also ISEVEN.

% By Ulf Carlberg.

if isfloat(x) && isreal(x)
    if isa(x, 'single')
        % The function "bitget" can not handle single, so we must check input
        % and use the function "mod".
        if any(floor(x(:))~=x(:))
            error('isodd:InputNotInt', 'All entries of X must be integers.');
        end
        if any(isnan(x(:)))
            error('isodd:InputNaN', 'No entries of X are allowed to be NaN.');
        end
        if any(x(:)>=2^24)  % also x=Inf
            error('isodd:InputOutOfRange', ...
                'The maximum value of X allowed is 2^24-1 for singles.');
        end
        if any(x(:)<=-2^24)  % also x=-Inf
            error('isodd:InputOutOfRange', ...
                'The minimum value of X allowed is -2^24+1 for singles.');
        end
        b = logical(mod(x, 2));
    else  % double
        if any(floor(x(:))~=x(:))
            % This test is not required in Matlab, but in Octave.
            error('isodd:InputNotInt', 'All entries of X must be integers.');
        end
        if any(x(:)>=2^53)  % also x=Inf
            error('isodd:InputOutOfRange', ...
                'The maximum value of X allowed is 2^53-1 for doubles.');
        end
        if any(x(:)<=-2^53)  % also x=-Inf
            error('isodd:InputOutOfRange', ...
                'The minimum value of X allowed is -2^53+1 for doubles.');
        end
        b = logical(bitget(abs(x), 1));
    end
elseif isinteger(x)
    b = logical(mod(x, 2));
else
    error('isodd:UnsupportedClass', 'Unsupported class.');
end

end
