function modify_image(in_filename, out_filename, crop_image, transparent_bg)
%MODIFY_IMAGE Crop PNG image file and/or define transparent background.
%
%   function modify_image(in_filename, out_filename, crop_image,
%                         transparent_bg)
%
%   Read the PNG image file "in_filename" and save it as "out_filename".
%   "crop_image" is a boolean (default true). "transparent_bg" is a
%   boolean (default false).
%
%   The function defines the background color as the color of the upper
%   left pixel. If cropping is selected it finds the smallest (unrotated)
%   rectangle with only background colors outside it. This rectangle
%   defines the new cropped image that is saved. If transparent background
%   ("transparent_bg") is set, it will also use the upper left pixel as
%   transparent color in the saved file.

% By Ulf Carlberg.

% Default values of arguments.
if nargin<2 || isempty(out_filename), out_filename = in_filename; end
if nargin<3 || isempty(crop_image), crop_image = 1; end
if nargin<4 || isempty(transparent_bg), transparent_bg = 0; end
extra_pixels_to_keep = 10;

% Read file.
[x, map] = imread(in_filename);

if crop_image
    
    % Find where there is not background color.
    is_not_background = x(:,:,1)~=x(1,1,1);
    for ii = 2:size(x, 3)
        is_not_background = is_not_background | x(:,:,ii)~=x(1,1,ii);
    end

    % Calculate the boundaries for cropping.
    irow1 = find(sum(is_not_background,2), 1, 'first');
    irow2 = find(sum(is_not_background,2), 1, 'last');
    icol1 = find(sum(is_not_background,1), 1, 'first');
    icol2 = find(sum(is_not_background,1), 1, 'last');

    % Keep extra pixels around the image.
    irow1 = max(irow1-extra_pixels_to_keep, 1);
    irow2 = min(irow2+extra_pixels_to_keep, size(x,1));
    icol1 = max(icol1-extra_pixels_to_keep, 1);
    icol2 = min(icol2+extra_pixels_to_keep, size(x,2));
else
    irow1 = 1;
    irow2 = size(x, 1);
    icol1 = 1;
    icol2 = size(x, 2);
end

% Set background transparent?
if transparent_bg
    trans_color = double(squeeze(x(1,1,:)));
    if isa(x, 'uint8')
        trans_color = trans_color/255;
    end
else
    trans_color = [];
end

% Save image.
in_file_info = imfinfo(in_filename);
if isoctave() && isempty(map)
    imwrite(x(irow1:irow2,icol1:icol2,:), out_filename);
elseif isoctave()
    imwrite(x(irow1:irow2,icol1:icol2,:), map, out_filename);
elseif isempty(map)
    imwrite(x(irow1:irow2,icol1:icol2,:), out_filename, ...
        'Transparency', trans_color, ...
        'XResolution', in_file_info.XResolution, ...
        'YResolution', in_file_info.YResolution, ...
        'ResolutionUnit', in_file_info.ResolutionUnit);
else
    imwrite(x(irow1:irow2,icol1:icol2,:), map, out_filename, ...
        'Transparency', trans_color, ...
        'XResolution', in_file_info.XResolution, ...
        'YResolution', in_file_info.YResolution, ...
        'ResolutionUnit', in_file_info.ResolutionUnit);
end

end
