function plot_farfield(a, ifreq, xyz, r, range_dB, max_dB, pol, with_rings)
%PLOT_FARFIELD Plot far-field ("radiation pattern") of antenna.
%
%   function plot_farfield(a, ifreq, xyz, r, range_dB, max_dB, pol, with_rings)
%
%   Plot far-field function of antenna "a" for frequency "ifreq" (default 1) 
%   with radius "r" (default 1) in position "xyz" (default [0 0 0]).
%
%   "range_dB" (default 10) and "max_dB" (default []) specifies plot range. If
%   "max_dB" is empty, the maximum value is used. Plotting will be from
%   "max_dB" minus "range_dB" to "max_dB".
%
%   "pol" can be 'total' (default) for plotting both polarizations added
%   together, 'theta-phi' for plotting Gtheta and Gphi (in separate subplots)
%   or 'xPrim-yPrim' for plotting Gx' and Gy' according to Ludwig's third
%   definition.
%
%   If "with_rings" (default true) is true, colored axes and rings will be
%   plotted.
%
%   See also PLOT_FARFIELD_2D and PLOT_FARFIELD_INPOS.

% By Ulf Carlberg.

% Define optional input arguments if needed.
if nargin<2 || isempty(ifreq), ifreq = 1; end
if nargin<3 || isempty(xyz), xyz = [0 0 0]; end
if nargin<4 || isempty(r), r = 1; end
if nargin<5 || isempty(range_dB), range_dB = 10; end
if nargin<6, max_dB = []; end
if nargin<7 || isempty(pol), pol = 'total'; end
if nargin<8 || isempty(with_rings), with_rings = 1; end

% Define unit vector R in xyz-coordinates for far-field angles.
[th, ph] = ndgrid(a.theta, a.phi);
Eth = a.Eth(:,:,ifreq);
Eph = a.Eph(:,:,ifreq);
[x, y, z] = unit_vector_r(th, ph);

% Calculate power for the chosen plot format.
if strcmpi(pol, 'total')
    W1_dB = 10*log10(abs(Eth).^2 + abs(Eph).^2);
    W2_dB = [];
elseif strcmpi(pol, 'theta-phi')
    W1_dB = 20*log10(abs(Eth));
    W2_dB = 20*log10(abs(Eph));
elseif strcmpi(pol, 'xPrim-yPrim')
    W1_dB = 20*log10(abs( Eth.*cosd(ph) - Eph.*sind(ph) ));
    W2_dB = 20*log10(abs( Eth.*sind(ph) + Eph.*cosd(ph) ));
else
    error('Unknown polarization argument (pol).');
end
if isempty(max_dB)
    max_dB = max([W1_dB(:) ; W2_dB(:)]);
end
min_dB = max_dB - range_dB;

% Normalize [min_dB..max_dB] -> [0..r] range. Clip values below zero.
W1 = r * (W1_dB-min_dB) / range_dB;
W2 = r * (W2_dB-min_dB) / range_dB;
W1(W1<0) = 0;
W2(W2<0) = 0;

% Plot (one normal or two subplots).
if isempty(W2)
    local_plot_commands(xyz, x, y, z, W1, r, with_rings);
else
    subplot(1, 2, 1);
    local_plot_commands(xyz, x, y, z, W1, r, with_rings);
    subplot(1, 2, 2);
    local_plot_commands(xyz, x, y, z, W2, r, with_rings);
end

end

%-------------------------------------------------------------------------------

function local_plot_commands(xyz_trans, x, y, z, W, max_W, with_rings)
    surf(xyz_trans(1)+x.*W, xyz_trans(2)+y.*W, xyz_trans(3)+z.*W, W);
    shading interp;
    set(gca, 'Projection', 'perspective');
    caxis([0 max_W]);
    colormap(jet(256));
    axis off;
    axis image;
    view(22, 22);
    if with_rings
        plot_axes_and_rings(max_W, xyz_trans);
    end
end

%-------------------------------------------------------------------------------
