function plot_farfield_2D(a, ifreq, range_dB, normalize, phi_cut)
%PLOT_FARFIELD_2D Plot 2D far-field ("radiation pattern") of antenna.
%
%   function plot_farfield_2D(a, ifreq, range_dB, normalize, phi_cut)
%
%   Plot far-field function of antenna "a" for frequency "ifreq" (default 1). 
%
%   The parameter "range_dB" (default 10) specifies plot range. If "normalize"
%   (default true) is true, then plot normalized gain. The parameter "phi_cut"
%   (default 180) are usually 180 or 360 and defines where the cut is made.
%
%   See also PLOT_FARFIELD and PLOT_FARFIELD_INPOS.

% By Ulf Carlberg.

% Define optional input arguments if needed.
if nargin<2 || isempty(ifreq), ifreq = 1; end
if nargin<3 || isempty(range_dB), range_dB = 10; end
if nargin<4 || isempty(normalize), normalize = true; end
if nargin<5 || isempty(phi_cut), phi_cut = 180; end

% Take out total gain for selected frequency.
W_dB = 10*log10(abs(a.Eth(:,:,ifreq)).^2 + abs(a.Eph(:,:,ifreq)).^2);

% Normalize to proper range in [dB].
if normalize
    W_dB = W_dB - max(W_dB(:));
end
max_dB = max(W_dB(:));
min_dB = max_dB - range_dB;
W_dB(W_dB<min_dB) = min_dB;

% Take care of phi-cut.
[~,ii] = min(abs(a.phi-phi_cut));
phi = [a.phi(ii:end-1)-360 ; a.phi(1:ii)];
W_dB = [W_dB(:,ii:end-1) W_dB(:,1:ii)];

% Do a surface plot. Make sure theta=0 is on top of figure.
surf(phi, a.theta, W_dB);
colormap(jet(256));
caxis([min_dB max_dB]);
colorbar;
view(0, 90);
shading interp;
set(gca, 'XTick', phi(1):45:phi(end));
set(gca, 'YTick', 0:45:180);
set(gca, 'YDir', 'reverse');
set(gca, 'TickDir', 'out');
box on;
if normalize
    title('Relative gain [dB]');
else
    title('Realized gain [dBi]');
end    
xlabel('\phi-angle [deg]');
ylabel('\theta-angle [deg]');
axis image;

end
