function plot_polar_format(plot_type, r_range_dB, phi_range)
%PLOT_POLAR_FORMAT Polar plot format for theta-cut and phi-cut.
%
%   function plot_polar_format(plot_type, r_range_dB, phi_range_deg)
%
%   Draw format lines and circles for polar plots. "plot_type" should be 
%   'thetacut' or 'phicut'. "r_range_dB" is a vector with dB-values on grid
%   (default -20:5:10) and "phi_range" is a vector with degree values on grid.
%
%   See also PLOT_POLAR_FARFIELD_THETACUT and PLOT_POLAR_FARFIELD_PHICUT. 

% By Ulf Carlberg.

if nargin<2 || isempty(r_range_dB), r_range_dB = -20:5:10; end
if nargin<3 || isempty(phi_range), phi_range = -150:30:180; end

% Help data for plotting.
circles_rad = exp(1j*linspace(-pi, pi, 121));

% Plot radial lines and degree text.
for phi = phi_range
    if strcmpi(plot_type, 'thetacut')
        x = cosd(phi);
        y = sind(phi);
    elseif strcmpi(plot_type, 'phicut')
        x = sind(phi);
        y = cosd(phi);
    else
        error('Unknown plot type.');
    end
    plot([0 x], [0 y], '-', 'Color', 0.4*[1 1 1]);
    hold on;
    text(1.15*x, 1.15*y, sprintf('%.0f^o', phi), 'FontSize', 6, ...
        'HorizontalAlignment', 'center');
end

% Plot circles.
for ii = 2:numel(r_range_dB)
    r = (r_range_dB(ii)-r_range_dB(1)) / (r_range_dB(end)-r_range_dB(1));
    plot(real(r*circles_rad), imag(r*circles_rad), '-', 'Color', 0.4*[1 1 1]);
    if any(ii==numel(r_range_dB)-(0:2:20))
        text(0, r, sprintf('%.0f dB', r_range_dB(ii)), ...
            'FontSize', 6, 'HorizontalAlignment', 'center', ...
            'BackgroundColor', [0.7 1 0.7]);
    end
end

% Format plot.
set(gca, 'xtick', []);
set(gca, 'ytick', []);
axis image;
axis(1.3*[-1 1 -1 1]);

% Save plot type and range to userdata.
h = gcf();
userdata.plot_type = plot_type;
userdata.min_dB = min(r_range_dB);
userdata.max_dB = max(r_range_dB);
set(h, 'UserData', userdata);

end
