function plot_waves(w, ir, enumerate_waves, scale_factor, patch_color)
%PLOT_WAVES Plot incoming plane waves as patches.
%
%   function plot_waves(w, ir, enumerate_waves, scale_factor, patch_color)
%
%   Plot waves of one realization. "w" is the waves and "ir" is the realization
%   number (optional, default 1). if "enumerate_waves" is true (default false)
%   each wave number is written out in text. "scale_factor" (optional, 
%   default 1) can be use to scale the plot geometry. "patch_color"
%   (default 'r') is the color of patches.

% By Ulf Carlberg.

% Define optional arguments.
if nargin<2 || isempty(ir), ir = 1; end
if nargin<3 || isempty(enumerate_waves), enumerate_waves = 0; end
if nargin<4 || isempty(scale_factor), scale_factor = 1; end
if nargin<5 || isempty(patch_color), patch_color = 'r'; end
s = scale_factor;  % less typing...

% Patch position radius and patch width.
rp = 10 + angle(w.Eth(ir,:).') / (2*pi);
d = 1;

% xyz-coordinates for the center of patches:
[xp, yp, zp] = rthetaphi2xyz(rp, w.theta(ir,:)', w.phi(ir,:)');

% xyz-coordinates of spherical unit vectors at patches.
[uTx, uTy, uTz] = unit_vector_theta(w.theta(ir,:)', w.phi(ir,:)');
[uPx, uPy, uPz] = unit_vector_phi(w.phi(ir,:)');

% Patches spanned by vectors [xp1, yp1, zp1] and [xp2, yp2, zp2]:
xp1 = d/2 * uTx;
yp1 = d/2 * uTy;
zp1 = d/2 * uTz;
xyz2 = cross([xp,yp,zp], [xp1,yp1,zp1], 2) ./ repmat(rp,1,3);
xp2 = xyz2(:,1);
yp2 = xyz2(:,2);
zp2 = xyz2(:,3);

% Plot the patches.
hold off;
patch(s*[xp-xp1-xp2 xp+xp1-xp2 xp+xp1+xp2 xp-xp1+xp2]', ...
      s*[yp-yp1-yp2 yp+yp1-yp2 yp+yp1+yp2 yp-yp1+yp2]', ...
      s*[zp-zp1-zp2 zp+zp1-zp2 zp+zp1+zp2 zp-zp1+zp2]', patch_color);
hold on;

% Calculate and plot polarization ellipses on patches.
norm_factor = 0.5*d / max(abs([w.Eth(:) ; w.Eph(:)]));
Eth = norm_factor * w.Eth(ir,:).';
Eph = norm_factor * w.Eph(ir,:).';
tmp = exp(1j*linspace(0, 2*pi, 17));
xo1 = xp + real((uTx.*Eth)*tmp) + real((uPx.*Eph)*tmp);
yo1 = yp + real((uTy.*Eth)*tmp) + real((uPy.*Eph)*tmp);
zo1 = zp + real((uTz.*Eth)*tmp) + real((uPz.*Eph)*tmp);
plot3(s*xo1.', s*yo1.', s*zo1.', 'k-', 'linewidth', 1);

% Plot coordinate axis.
plot3([0 3*s ; 0 0 ; 0 0]', [0 0 ; 0 3*s ; 0 0]', [0 0 ; 0 0 ; 0 3*s]', 'k-');
text(4*s, 0, 0, 'x');
text(0, 4*s, 0, 'y');
text(0, 0, 4*s, 'z');

% Show wave numbers.
if enumerate_waves
    for iw = 1:numel(xp)
        text(s*xp(iw), s*yp(iw), s*zp(iw), num2str(iw));
    end
end

% Define figure settings.
set(gca, 'Projection', 'perspective');
colormap(jet(256));
axis image;
axis off;
view(22, 22);
axis(10*s*[-1 1 -1 1 -1 1]);

end
