function a = read_farfield(filename)
%READ_FARFIELD Read antenna far-fields from ASCII files.
%
%   function a = read_farfield(filename)
%
%   Read far-field from an ASCII file and output an antenna structure. This
%   function can handle several different formats.

% By Ulf Carlberg.

% Use this list of first lines to detect file format.
firstlines = {...
    '// CST Farfield Source File', ...
    'Theta[deg.]  Phi[deg.]  Abs(Dir.)[dBi] Abs(Theta)[dBi]', ...
    'frequency phi theta Ephi_amp_lin Etheta_amp_lin Ephi_phase_rad', ...
    'Phi[deg],Theta[deg],dBc(RealizedGainTotal)', ...
    'Phi[deg],Theta[deg],mag(RealizedGainTotal)', ...
    ['Phi[deg],Theta[deg],mag(rEPhi)[mV],ang_deg(rEPhi)[mV],' ...
        'mag(rETheta)[mV],ang_deg(rETheta)[mV]'], ...
    ['Phi[deg],Theta[deg],re(rEPhi)[mV],im(rEPhi)[mV],' ...
        're(rETheta)[mV],im(rETheta)[mV]'], ...
    'Phi[deg],Theta[deg],dB(RealizedGainTotal)', ...
    '"Theta [deg]","mag(rEPhi) [V] - Freq=', ...
    '"Theta [deg]","re(rEPhi) [V] - Freq=', ...
    };

% Read first line of ASCII file to detect file format.
f = fopen(filename);
firstline = fgetl(f);
s = firstline(firstline~=' ');
fclose(f);
fileformat = 0;
for ii = 1:numel(firstlines)
    tmp = firstlines{ii};
    tmp = tmp(tmp~=' ');
    if strncmpi(tmp, s, numel(tmp))
        fileformat = ii;
        break;
    end
end

% Define antenna structure.
a.type = 'farfield';
a.name = ['File: ' filename];
a.actions = {sprintf('read_farfield(''%s'')', filename)};

if fileformat == 1
    str_ver = '// Version:';
    str_freq = '// Frequency';
    str_nfreq = '// #Frequencies';
    str_freq1 = '// Radiated/Accepted/Stimulated Power , Frequency';
    str_nang = '// >> Total #phi samples, total #theta samples';
    str_data = '// >> Phi, Theta, Re(E_Theta), Im(E_Theta), Re(E_Phi),';
    f = fopen(filename);
    s = '';
    while ~strncmpi(s, str_ver, numel(str_ver))
        s = fgetl(f);
    end
    ver = str2double(fgetl(f));
    if ver==2.1
        Nfreq = 1;
        while ~strncmpi(s, str_freq, numel(str_freq))
            s = fgetl(f);
        end
        freq = str2double(fgetl(f));
        while ~strncmpi(s, str_data, numel(str_data))
            s = fgetl(f);
        end
        rawdata = textscan(f, '%f');
        fclose(f);
        data = reshape(rawdata{1}, 6, [])';
        phi2D = data(:,1);
        Nth = find(phi2D~=phi2D(1),1) - 1;
        Nph = length(phi2D)/Nth;
        phi2D = reshape(phi2D, Nth, Nph);
        theta2D = reshape(data(:,2), Nth, Nph);
        sf = sqrt(2*pi/eta0);  % scale factor
        Eth = sf * reshape(complex(data(:,3),data(:,4)), Nth, Nph);
        Eph = sf * reshape(complex(data(:,5),data(:,6)), Nth, Nph);
    elseif ver==3.0
        while ~strncmpi(s, str_nfreq, numel(str_nfreq))
            s = fgetl(f);
        end
        Nfreq = str2double(fgetl(f));
        while ~strncmpi(s, str_freq1, numel(str_freq1))
            s = fgetl(f);
        end
        tmp = textscan(f, '%f', 4*Nfreq);
        freq = tmp{1}(4:4:end);
        for ifreq = 1:Nfreq
            while ~strncmpi(s, str_nang, numel(str_nang))
                s = fgetl(f);
            end
            tmp = textscan(f, '%f', 2);
            Nph = tmp{1}(1);
            Nth = tmp{1}(2);
            while ~strncmpi(s, str_data, numel(str_data))
                s = fgetl(f);
            end
            rawdata = textscan(f, '%f', 6*Nph*Nth);
            data = reshape(rawdata{1}, 6, [])';
            phi2D = data(:,1);
            phi2D = reshape(phi2D, Nth, Nph);
            theta2D = reshape(data(:,2), Nth, Nph);
            sf = sqrt(2*pi/eta0);  % scale factor
            Eth(:,:,ifreq) = sf * reshape(complex(data(:,3),data(:,4)), Nth, Nph);
            Eph(:,:,ifreq) = sf * reshape(complex(data(:,5),data(:,6)), Nth, Nph);
        end
        fclose(f);
    end
elseif fileformat == 2
    data = read_ascii_file(filename, 2);
    freq = -1;
    phi2D = data(:,2);
    Nth = find(phi2D~=phi2D(1), 1, 'first') - 1;
    Nph = numel(phi2D) / Nth;
    phi2D = reshape(phi2D, Nth, Nph);
    theta2D = reshape(data(:,1), Nth, Nph);
    if size(data,2)==8  % 8 columns => [dB] units
        Eth_mag = 10.^(data(:,4)/20);
        Eph_mag = 10.^(data(:,6)/20);
    elseif size(data,2)==9  % 9 columns => [V/m] units
        Eth_mag = data(:,4);
        Eph_mag = data(:,6);
    end
    Eth = reshape(Eth_mag.*exp(1j*data(:,5)*pi/180), Nth, Nph);
    Eph = reshape(Eph_mag.*exp(1j*data(:,7)*pi/180), Nth, Nph);
elseif fileformat == 3
    data = read_ascii_file(filename, 1);
    npoints_per_freq = find(data(:,1)~=data(1,1), 1, 'first') - 1;
    freq = data(1:npoints_per_freq:end,1);
    phi2D = data(1:npoints_per_freq,2)*180/pi;
    Nth = find(phi2D~=phi2D(1), 1, 'first') - 1;
    Nph = numel(phi2D) / Nth;
    phi2D = reshape(phi2D, Nth, Nph);
    theta2D = reshape(data(1:npoints_per_freq,3), Nth, Nph)*180/pi;
    Eth = reshape(data(:,5).*exp(1j*data(:,7)), Nth, Nph, numel(freq));
    Eph = reshape(data(:,4).*exp(1j*data(:,6)), Nth, Nph, numel(freq));
elseif fileformat == 4 || fileformat == 5
    data = read_ascii_file(filename, 1);
    freq = -1;
    theta2D = data(:,2);
    Nph = find(theta2D~=theta2D(1), 1, 'first') - 1;
    Nth = numel(theta2D) / Nph;
    phi2D = reshape(data(:,1), Nph, Nth)';
    theta2D = reshape(theta2D, Nph, Nth)';
    if fileformat == 4
        G_dB = reshape(data(:,3), Nph, Nth)';
    elseif fileformat == 5
        G_dB = 20*log10(reshape(data(:,3), Nph, Nth)');
    end
    
    % Ignore redundant data.
    if abs(phi2D(1,1)-phi2D(1,end)) == 360 && ...
            abs(theta2D(1,1)-theta2D(end,1)) == 360
        warning(['Ignoring redundant data in "' filename '".']);
        i1 = find(theta2D(:,1)==0, 1, 'first');
        phi2D = phi2D(i1:end,:);
        theta2D = theta2D(i1:end,:);
        G_dB = G_dB(i1:end,:);
    end
    
    % Convert format to positive theta angles.
    if any(theta2D(:,1)<0)
        i1 = find(theta2D(:,1)==0, 1, 'first');
        phi2D = [phi2D(i1:end,:) flipud(phi2D(1:i1,2:end)+180)];
        theta2D = [theta2D(i1:end,:) flipud(-theta2D(1:i1,2:end))];
        G_dB = [G_dB(i1:end,:) flipud(G_dB(1:i1,2:end))];
    end

    % Assume x'-polarized.
    ExPrim = 10.^(G_dB/20);
    Eth = +cosd(phi2D) .* ExPrim;
    Eph = -sind(phi2D) .* ExPrim;
elseif fileformat == 6
    data = read_ascii_file(filename, 1);
    freq = -1;
    theta2D = data(:,2);
    Nph = find(theta2D~=theta2D(1), 1, 'first') - 1;
    phi2D = reshape(data(:,1), Nph, [])';
    theta2D = reshape(data(:,2), Nph, [])';
    sf1 = sqrt(2*pi/eta0) / 1000;  % scale factor 1
    sf2 = 1j / 1000;  % scale factor 2
    Eph = reshape(sf1*data(:,3).*exp(sf2*data(:,4)), Nph, []).';
    Eth = reshape(sf1*data(:,5).*exp(sf2*data(:,6)), Nph, []).';
elseif fileformat == 7
    data = read_ascii_file(filename, 1);
    freq = -1;
    Nph = find(data(:,2)~=data(1,2), 1, 'first') - 1;
    phi2D = reshape(data(:,1), Nph, [])';
    theta2D = reshape(data(:,2), Nph, [])';
    sf = sqrt(2*pi/eta0) / 1000;  % scale factor
    Eph = sf * reshape(complex(data(:,3),data(:,4)), Nph, []).';
    Eth = sf * reshape(complex(data(:,5),data(:,6)), Nph, []).';
elseif fileformat == 8
    data = read_ascii_file(filename, 1);
    freq = -1;
    Nph = find(data(:,2)~=data(1,2), 1, 'first') - 1;
    phi2D = reshape(data(:,1), Nph, [])';
    theta2D = reshape(data(:,2), Nph, [])';
    Eth = reshape(10.^(data(:,3)/20), Nph, [])';
    Eph = zeros(size(Eth));
elseif fileformat == 9 || fileformat == 10
    data = read_ascii_file(filename, 1);
    s = strrep(firstline, ' - ', ' ');
    d = sscanf(s(s==' ' | s=='.' | s=='-' | s>='0' & s<='9'), '%f');
    freq = 1e9 * d(1:2:end);
    phi = d(2:2:end);
    Nth = size(data, 1);
    Nph = find(phi(2:end)==phi(1), 1, 'first');
    Nfreq = numel(freq) / (4*Nph);
    [theta2D,phi2D] = ndgrid(data(:,1),phi(1:Nph));
    freq = freq(1:Nph:Nph*Nfreq);
    Eth = zeros(Nth, Nph, Nfreq);
    Eph = zeros(Nth, Nph, Nfreq);
    indx = 1 + (1:Nph);
    for ifreq = 1:Nfreq
        d1 = data(:,indx);
        d2 = data(:,Nph*Nfreq+indx);
        d3 = data(:,2*Nph*Nfreq+indx);
        d4 = data(:,3*Nph*Nfreq+indx);
        if fileformat == 9
            Eph(:,:,ifreq) = sqrt(2*pi/eta0) * d1 .* exp(1j*d2*pi/180);
            Eth(:,:,ifreq) = sqrt(2*pi/eta0) * d3 .* exp(1j*d4*pi/180);
        elseif fileformat == 10
            Eph(:,:,ifreq) = sqrt(2*pi/eta0) * complex(d1, d2);
            Eth(:,:,ifreq) = sqrt(2*pi/eta0) * complex(d3, d4);
        end
        indx = indx + Nph;
    end
else
    error('Unknown file format.');
end

% Make sure that phi is in the interval [0..360].
tol = 1e-3;
i1 = find(phi2D(1,:)>-tol, 1, 'first');
i2 = size(phi2D,2);
if abs(sind(phi2D(1,1))-sind(phi2D(1,end))) > tol
    i3 = 1;
else
    i3 = 2;
end
phi2D = mod([phi2D(:,i1:i2,:) phi2D(:,i3:i1,:)], 360);
phi2D(:,end,:) = 360;
Eth = [Eth(:,i1:i2,:) Eth(:,i3:i1,:)];
Eph = [Eph(:,i1:i2,:) Eph(:,i3:i1,:)];

% Save data in output structure.
a.freq = freq;
a.Nth = size(Eth, 1);
a.Nph = size(Eth, 2);
a.Nfreq = numel(a.freq);
a.theta = theta2D(:,1);
a.phi = phi2D(1,:)';
a.Eth = single(Eth);  % single precision is more than enough and saves memory
a.Eph = single(Eph);

% Check angles and make sure they are exact.
assert(abs(a.theta(1))<=tol, 'theta angle wrong');
assert(abs(a.theta(end)-180)<=tol, 'theta angle wrong');
assert(abs(a.phi(1))<=tol, 'phi angle wrong');
assert(abs(a.phi(end)-360)<=tol, 'phi angle wrong');
a.theta(1) = 0;
a.theta(end) = 180;
a.phi(1) = 0;
a.phi(end) = 360;

end
