function a = rotate_farfield(a, rot_alpha, rot_theta, rot_phi, Ntheta, Nphi, interp_method)
%ROTATE_FARFIELD Rotation of antenna far-field structure with Euler angles.
%
%   function a = rotate_farfield(a, rot_alpha, rot_theta, rot_phi, ...
%       Ntheta, Nphi, interp_method)
%
%   Rotate antenna structure "a" and output a new antenna structure. The
%   rotations are done around the z-axis with angle "rot_alpha", then around the
%   y-axis with rotation angle "rot_theta", and finally around the z-axis
%   (again) with rotation angle "rot_phi".
%
%   The optional arguments "Ntheta" and "Nphi" can be used to resample the
%   far-field function.

% By Ulf Carlberg.

if nargin<5 || isempty(Ntheta), Ntheta = a.Nth; end
if nargin<6 || isempty(Nphi), Nphi = a.Nph; end
if nargin<7 || isempty(interp_method)
    % Octave can't handle 'spline' interpolation in interp2 for non-uniform
    % grid.
    if isoctave()
        interp_method = 'linear';
    else
        interp_method = 'spline';
    end
end

% Resample angles?
if Ntheta ~= a.Nth
    theta = linspace(0, 180, Ntheta)';
else
    theta = a.theta;
end
if Nphi ~= a.Nph
    phi = linspace(0, 360, Nphi)';
else
    phi = a.phi;
end

% Leave tag on structure.
a.actions{end+1,1} = ['rotate_farfield(*, ' function_call_str({rot_alpha, ...
    rot_theta, rot_phi, Ntheta, Nphi, interp_method}) ')'];

% Define the 2D grid of theta and phi for the new rotated far-field.
[theta2D, phi2D] = ndgrid(theta, phi);

% Calculate the spherical unit vectors for the far-field of the new rotated
% far-field.
[uRx, uRy, uRz] = unit_vector_r(theta2D, phi2D, 1);
[uTx, uTy, uTz] = unit_vector_theta(theta2D, phi2D);
[uPx, uPy, ~] = unit_vector_phi(phi2D);

% Rotate the unit vectors R backwards to get the theta and phi angles of the old
% unrotated antenna structure.
[uRx2, uRy2, uRz2] = zyzrot(uRx, uRy, uRz, -rot_phi, -rot_theta, -rot_alpha);
[~, theta2D_old, phi2D_old] = xyz2rthetaphi(uRx2, uRy2, uRz2);

% Calculate unit vectors in old directions.
[uTx2, uTy2, uTz2] = unit_vector_theta(theta2D_old, phi2D_old);
[uPx2, uPy2, uPz2] = unit_vector_phi(phi2D_old);

% Rotate the new unit vectors forwards.
[uTx2, uTy2, uTz2] = zyzrot(uTx2, uTy2, uTz2, rot_alpha, rot_theta, rot_phi);
[uPx2, uPy2, uPz2] = zyzrot(uPx2, uPy2, uPz2, rot_alpha, rot_theta, rot_phi);

% Scalar products.
dp_tt = uTx.*uTx2 + uTy.*uTy2 + uTz.*uTz2;
dp_tp = uTx.*uPx2 + uTy.*uPy2 + uTz.*uPz2;
dp_pt = uPx.*uTx2 + uPy.*uTy2;
dp_pp = uPx.*uPx2 + uPy.*uPy2;

% Loop through frequencies.
Eth_rot = zeros(Ntheta, Nphi, a.Nfreq);
Eph_rot = zeros(Ntheta, Nphi, a.Nfreq);
for ifreq = 1:a.Nfreq

    % Calculate the field in new directions.
    Eth = interp2(a.phi, a.theta, a.Eth(:,:,ifreq), ...
        phi2D_old, theta2D_old, interp_method, NaN);
    Eph = interp2(a.phi, a.theta, a.Eph(:,:,ifreq), ...
        phi2D_old, theta2D_old, interp_method, NaN);

    % Project the far-field defined with the rotated unit vectors onto the
    % proper unit vectors.
    Eth_rot(:,:,ifreq) = Eth.*dp_tt + Eph.*dp_tp;
    Eph_rot(:,:,ifreq) = Eth.*dp_pt + Eph.*dp_pp;
end

% Update structure that is returned.
a.Eth = single(Eth_rot);
a.Eph = single(Eph_rot);
a.Nth = Ntheta;
a.Nph = Nphi;
a.theta = theta;
a.phi = phi;

end
