function [theta, phi] = sph_regsamp(N)
%SPH_REGSAMP Regular sampling of the unit sphere.
%
%   function [theta, phi] = sph_regsamp(N)
%
%   Regular sampling of the unit sphere with "N" number of samples. The function
%   defines a spiral from the north pole to the south pole with consant arc
%   lengths between samples. Samples are never exactly on the poles. Angles are
%   given in degrees.

% By Ulf Carlberg.

% Trivial case. Propagate correct zero-sized matrices.
if N<1
    theta = zeros(0, 1);
    phi = zeros(0, 1);
    return;
end

% Sample theta so that each point have a constant delta area. (Start and end
% with half area.)
s = linspace(0, 2, 2*N+1)';
theta = acosd(1-s(2:2:2*N));

% Sample phi so that the distance between points in the spiral is constant. Use
% the inverse of the Haversine formula.
dl = sqrt(4*pi/N);
th1 = theta(1:N-1);
th2 = theta(2:N);
tmp = ( sin(dl/2)^2 - sind((th2-th1)/2).^2 ) ./ ( sind(th1) .* sind(th2) );
dphi = 2 * asind(sqrt(tmp));

% Define phi angles in interval [0..360] starting with 0.
phi = mod([0 ; cumsum(dphi)], 360);

end
