function [xr, yr, zr] = zyzrot(x, y, z, alpha, theta, phi)
%ZYZROT Rotate coordinates with Euler angles (z-y-z fixed axes).
%
%   function [xr, yr, zr] = zyzrot(x, y, z, alpha, theta, phi)
%
%   Rotates objects with coordinates "x", "y", and "z" around the fixed unit
%   axes.
%   1. Rotate around the (fixed) z-axis with "alpha" degrees.
%   2. Rotate around the (fixed) y-axis with "theta" degrees.
%   3. Rotate around the (fixed) z-axis (again) with "phi" degrees.
%
%   This makes it possible to rotate an object to any orientation. Gimbal lock
%   occurs when "theta" is an integer multiple of 180 degrees, but this is not a
%   problem. Note that the rotation angles differ from most definitions, since
%   fixed axis are used. (The "theta" and "phi" angles are similar to the angles
%   used in spherical coordinatinates.)

% By Ulf Carlberg.

% Calculate sinus and cosinus of angles (given in degrees).
c1 = cosd(alpha);
s1 = sind(alpha);
c2 = cosd(theta);
s2 = sind(theta);
c3 = cosd(phi);
s3 = sind(phi);

% Calculate matrix coefficients.
axx = c1.*c2.*c3 - s1.*s3;
axy = -c1.*s3 - c2.*c3.*s1;
axz = c3.*s2;
ayx = c1.*c2.*s3 + c3.*s1;
ayy = c1.*c3 - c2.*s1.*s3;
ayz = s2.*s3;
azx = -c1.*s2;
azy = s1.*s2;
azz = c2;

% Rotation matrix implementation.
xr = axx.*x + axy.*y + axz.*z;
yr = ayx.*x + ayy.*y + ayz.*z;
zr = azx.*x + azy.*y + azz.*z;

end
